"===============================================================================
"
"          File:  textfilter.vim
"
"   Description:  Several text filters.
"
"   VIM Version:  7.0+
"        Author:  Dr. Fritz Mehner (fgm), mehner.fritz@web.de
"  Organization:
"       Version:  see variable  g:TextfilterVersion  below
"       Created:  29.07.2004 - 18:32:34
"      Revision:  ---
"       License:  Copyright (c) 2004-2018, Fritz Mehner
"                 This program is free software; you can redistribute it and/or
"                 modify it under the terms of the GNU General Public License as
"                 published by the Free Software Foundation, version 2 of the
"                 License.
"                 This program is distributed in the hope that it will be
"                 useful, but WITHOUT ANY WARRANTY; without even the implied
"                 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
"                 PURPOSE.
"                 See the GNU General Public License version 2 for more details.
"
"===================================================================================
"
if exists("g:TextfilterVersion") || &cp
  finish                                        " Prevent duplicate loading
endif
let g:TextfilterVersion= "2.0"      " version number of this script; do not change
"
"===================================================================================
"  DEFAULTS         {{{1
"===================================================================================
let s:Filter_FormatDate         = '%x'
let s:Filter_FormatTime         = '%X'
let s:Filter_HighlightAfterCol  = '80'
let s:Filter_LoadMenus          = 'yes'            " toggle default
let s:Filter_RootMenu           = '&Textfilter.'   " name of the root menu (not empty)
let s:Filter_TimestampFormat    = '%y%m%d.%H%M%S'
"
let s:Filter_ColumnateInputSep  = ' '              " single blank
let s:Filter_ColumnateLeft      = 'yes'            " toggle default
let s:Filter_ColumnateOutputSep = ' '              " single blank
let s:Filter_EscFilename        = ' \%#[]'         " characters that must be escaped for filenames
let s:Filter_MenuVisible        = -1               " state variable controlling the C-menus
let s:Filter_NlBlankLines       = 'yes'            " toggle default
let s:Filter_NlIncrement        = 1                " line number increment
let s:Filter_NlLeadingZeros     = 'yes'            " toggle default
let s:Filter_NlStart            = 1                " starting line number
let s:Filter_NlWidth            = 0                " 0 = auto
let s:Filter_SeqOfNumCmd        = ""               " the last command
let s:Filter_SortAlpha          = 'yes'            " toggle default
let s:Filter_SortForwards       = 'yes'            " toggle default
let s:Filter_SortNotUnique      = 'yes'            " toggle default
let s:Filter_SplitSeparator     = '\s\+'           " one or more blanks
"
"===================================================================================
" MENUS         {{{1
"===================================================================================
"
"------------------------------------------------------------------------------
"  FilterInitMenu : initialize menus          {{{2
"------------------------------------------------------------------------------
function! FilterInitMenu ()
  "
  " -----  sort lines of text files  -------------------------
  "
  exe "amenu  <silent> .100 ".s:Filter_RootMenu."&sort\\ lines\\              :call FilterSort('a')<CR>"
  exe "imenu  <silent> .100 ".s:Filter_RootMenu."&sort\\ lines\\         <Esc>:call FilterSort('a')<CR>i"
  exe "vmenu  <silent> .100 ".s:Filter_RootMenu."&sort\\ lines\\         <Esc>:call FilterSort('v')<CR>"
  "
  if s:Filter_SortForwards=='yes'
    exe "amenu  <silent> .105 ".s:Filter_RootMenu."-\\ &forwards->reverse       :call FilterSortForwardsToggle()<CR>"
    exe "imenu  <silent> .105 ".s:Filter_RootMenu."-\\ &forwards->reverse  <Esc>:call FilterSortForwardsToggle()<CR>i"
    exe "vmenu  <silent> .105 ".s:Filter_RootMenu."-\\ &forwards->reverse  <Esc>:call FilterSortForwardsToggle()<CR>gv"
  else
    exe "amenu  <silent> .105 ".s:Filter_RootMenu."-\\ reverse->&forwards       :call FilterSortForwardsToggle()<CR>"
    exe "imenu  <silent> .105 ".s:Filter_RootMenu."-\\ reverse->&forwards  <Esc>:call FilterSortForwardsToggle()<CR>i"
    exe "vmenu  <silent> .105 ".s:Filter_RootMenu."-\\ reverse->&forwards  <Esc>:call FilterSortForwardsToggle()<CR>gv"
  endif
  "
  if s:Filter_SortAlpha=='yes'
    exe "amenu  <silent> .110 ".s:Filter_RootMenu."-\\ &alpha->numeric          :call FilterSortNumericToggle()<CR>"
    exe "imenu  <silent> .110 ".s:Filter_RootMenu."-\\ &alpha->numeric     <Esc>:call FilterSortNumericToggle()<CR>i"
    exe "vmenu  <silent> .110 ".s:Filter_RootMenu."-\\ &alpha->numeric     <Esc>:call FilterSortNumericToggle()<CR>gv"
  else
    exe "amenu  <silent> .110 ".s:Filter_RootMenu."-\\ numeric->&alpha          :call FilterSortNumericToggle()<CR>"
    exe "imenu  <silent> .110 ".s:Filter_RootMenu."-\\ numeric->&alpha     <Esc>:call FilterSortNumericToggle()<CR>"
    exe "vmenu  <silent> .110 ".s:Filter_RootMenu."-\\ numeric->&alpha     <Esc>:call FilterSortNumericToggle()<CR>gv"
  endif
  "
  if s:Filter_SortNotUnique=='yes'
    exe "amenu  <silent> .115 ".s:Filter_RootMenu."-\\ not\\ unique->&unique        :call FilterSortUniqToggle()<CR>"
    exe "imenu  <silent> .115 ".s:Filter_RootMenu."-\\ not\\ unique->&unique   <Esc>:call FilterSortUniqToggle()<CR>i"
    exe "vmenu  <silent> .115 ".s:Filter_RootMenu."-\\ not\\ unique->&unique   <Esc>:call FilterSortUniqToggle()<CR>gv"
  else
    exe "amenu  <silent> .115 ".s:Filter_RootMenu."-\\ &unique->not\\ unique        :call FilterSortUniqToggle()<CR>"
    exe "imenu  <silent> .115 ".s:Filter_RootMenu."-\\ &unique->not\\ unique   <Esc>:call FilterSortUniqToggle()<CR>i"
    exe "vmenu  <silent> .115 ".s:Filter_RootMenu."-\\ &unique->not\\ unique   <Esc>:call FilterSortUniqToggle()<CR>gv"
  endif
  "
  exe "amenu  <silent>  .120  ".s:Filter_RootMenu."-SEP0-            :"
  "
  " -----  number lines  -------------------------
  "
  exe "amenu <silent>  .205  ".s:Filter_RootMenu."&number\\ lines         :call FilterNumberLines('a')<CR>"
  exe "imenu <silent>  .205  ".s:Filter_RootMenu."&number\\ lines    <Esc>:call FilterNumberLines('a')<CR>i"
  exe "vmenu <silent>  .205  ".s:Filter_RootMenu."&number\\ lines    <Esc>:call FilterNumberLines('v')<CR>"
  "
  if s:Filter_NlLeadingZeros=='yes'
    exe "amenu <silent> .205 ".s:Filter_RootMenu."-\\ lead\\.&zeros->no\\ lead\\.zeros        :call FilterNumberLineZeroToggle()<CR>"
    exe "imenu <silent> .205 ".s:Filter_RootMenu."-\\ lead\\.&zeros->no\\ lead\\.zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>i"
    exe "vmenu <silent> .205 ".s:Filter_RootMenu."-\\ lead\\.&zeros->no\\ lead\\.zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>gv"
  else
    exe "amenu <silent> .205 ".s:Filter_RootMenu."-no\\ \\ lead\\.zeros->lead\\.&zeros        :call FilterNumberLineZeroToggle()<CR>"
    exe "imenu <silent> .205 ".s:Filter_RootMenu."-no\\ \\ lead\\.zeros->lead\\.&zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>i"
    exe "vmenu <silent> .205 ".s:Filter_RootMenu."-no\\ \\ lead\\.zeros->lead\\.&zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>gv"
  endif
  "
  exe "amenu <silent> .206 ".s:Filter_RootMenu."-\\ number\\ &width       :call FilterNumberLinesWidth()<CR>"
  exe "imenu <silent> .206 ".s:Filter_RootMenu."-\\ number\\ &width  <Esc>:call FilterNumberLinesWidth()<CR>i"
  exe "vmenu <silent> .206 ".s:Filter_RootMenu."-\\ number\\ &width  <Esc>:call FilterNumberLinesWidth()<CR>gv"
  "
  exe "amenu <silent> .207 ".s:Filter_RootMenu."-\\ &line\\ number\\ increment       :call FilterNumberIncrement()<CR>"
  exe "imenu <silent> .207 ".s:Filter_RootMenu."-\\ &line\\ number\\ increment  <Esc>:call FilterNumberIncrement()<CR>i"
  exe "vmenu <silent> .207 ".s:Filter_RootMenu."-\\ &line\\ number\\ increment  <Esc>:call FilterNumberIncrement()<CR>gv"
  "
  exe "amenu <silent> .208 ".s:Filter_RootMenu."-\\ &starting\\ line\\ number       :call FilterNumberLinesStart()<CR>"
  exe "imenu <silent> .208 ".s:Filter_RootMenu."-\\ &starting\\ line\\ number  <Esc>:call FilterNumberLinesStart()<CR>i"
  exe "vmenu <silent> .208 ".s:Filter_RootMenu."-\\ &starting\\ line\\ number  <Esc>:call FilterNumberLinesStart()<CR>gv"
  "
  if s:Filter_NlBlankLines == 'yes'
    exe "amenu <silent> .209 ".s:Filter_RootMenu."-\\ &blank\\ lines->no\\ blank\\ lines        :call FilterNumberLineBlankLinesToggle()<CR>"
    exe "imenu <silent> .209 ".s:Filter_RootMenu."-\\ &blank\\ lines->no\\ blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>i"
    exe "vmenu <silent> .209 ".s:Filter_RootMenu."-\\ &blank\\ lines->no\\ blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>gv"
  else
    exe "amenu <silent> .209 ".s:Filter_RootMenu."-\\ no\\ &blank\\ lines->blank\\ lines        :call FilterNumberLineBlankLinesToggle()<CR>"
    exe "imenu <silent> .209 ".s:Filter_RootMenu."-\\ no\\ &blank\\ lines->blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>i"
    exe "vmenu <silent> .209 ".s:Filter_RootMenu."-\\ no\\ &blank\\ lines->blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>gv"
  endif
  "
  exe "amenu <silent>  .210 ".s:Filter_RootMenu."-SEP1-              :"
  "
  " -----  columnate lists  -------------------------
  "
  exe "amenu <silent> .300 ".s:Filter_RootMenu."&columnate\\ list         :call FilterColumnateLists('a')<CR>"
  exe "imenu <silent> .300 ".s:Filter_RootMenu."&columnate\\ list    <Esc>:call FilterColumnateLists('a')<CR>i"
  exe "vmenu <silent> .300 ".s:Filter_RootMenu."&columnate\\ list    <Esc>:call FilterColumnateLists('v')<CR>"
  "
  exe "amenu <silent> .305 ".s:Filter_RootMenu."-\\ input\\ s&eparator            :call FilterColumnateInputSep()<CR>"
  exe "imenu <silent> .305 ".s:Filter_RootMenu."-\\ input\\ s&eparator       <Esc>:call FilterColumnateInputSep()<CR>i"
  exe "vmenu <silent> .305 ".s:Filter_RootMenu."-\\ input\\ s&eparator       <Esc>:call FilterColumnateInputSep()<CR>gv"
  "
  exe "amenu <silent> .305 ".s:Filter_RootMenu."-\\ output\\ s&eparator            :call FilterColumnateOutputSep()<CR>"
  exe "imenu <silent> .305 ".s:Filter_RootMenu."-\\ output\\ s&eparator       <Esc>:call FilterColumnateOutputSep()<CR>i"
  exe "vmenu <silent> .305 ".s:Filter_RootMenu."-\\ output\\ s&eparator       <Esc>:call FilterColumnateOutputSep()<CR>gv"
  "
  exe "amenu <silent> .306 ".s:Filter_RootMenu."-\\ &left->right          :call FilterColumnateLeftRightToggle()<CR>"
  exe "imenu <silent> .306 ".s:Filter_RootMenu."-\\ &left->right     <Esc>:call FilterColumnateLeftRightToggle()<CR>i"
  exe "vmenu <silent> .306 ".s:Filter_RootMenu."-\\ &left->right     <Esc>:call FilterColumnateLeftRightToggle()<CR>gv"
  "
  exe "amenu <silent>  .310 ".s:Filter_RootMenu."-SEP2-              :"
  "
  " -----  expand  :  tabs -> spaces  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."&tabs\\ to\\ spaces            :call FilterExpand('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."&tabs\\ to\\ spaces       <Esc>:call FilterExpand('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."&tabs\\ to\\ spaces       <Esc>:call FilterExpand('v')<CR>"
  "
  " -----  unexpand  :  spaces -> tabs  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."s&paces\\ to\\ tabs            :call FilterUnexpand('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."s&paces\\ to\\ tabs       <Esc>:call FilterUnexpand('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."s&paces\\ to\\ tabs       <Esc>:call FilterUnexpand('v')<CR>"
  "
  exe "amenu <silent>  ".s:Filter_RootMenu."-SEP3-              :"
  "
  " -----  compress empty lines  --------------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."compress\\ empt&y\\ lines      :call FilterCompressEmptyLines('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."compress\\ empt&y\\ lines <Esc>:call FilterCompressEmptyLines('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."compress\\ empt&y\\ lines <Esc>:call FilterCompressEmptyLines('v')<CR>"
  "
  exe "amenu <silent>  ".s:Filter_RootMenu."-SEP4-              :"
  "
  " -----  fill lines to equal length with blanks  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."f&ill\\ lines\\ (spaces)       :call FilterMultiLineEndFill('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."f&ill\\ lines\\ (spaces)  <Esc>:call FilterMultiLineEndFill('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."f&ill\\ lines\\ (spaces)  <Esc>:call FilterMultiLineEndFill('v')<CR>"
  "
  " -----  trim trailing spaces  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."tri&m\\ trail\\.\\ whitespaces        :call FilterMultiLineTrim('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."tri&m\\ trail\\.\\ whitespaces   <Esc>:call FilterMultiLineTrim('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."tri&m\\ trail\\.\\ whitespaces   <Esc>:call FilterMultiLineTrim('v')<CR>"
  "
  exe "amenu <silent>  ".s:Filter_RootMenu."-SEP5-              :"
  "
  " -----  left justify lines  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."&left\\ justify         :call FilterLeftJustify('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."&left\\ justify    <Esc>:call FilterLeftJustify('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."&left\\ justify    <Esc>:call FilterLeftJustify('v')<CR>"
  "
  " -----  right justify lines  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."&right\\ justify        :call FilterRightJustify('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."&right\\ justify   <Esc>:call FilterRightJustify('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."&right\\ justify   <Esc>:call FilterRightJustify('v')<CR>"
  "
  exe "amenu <silent>  ".s:Filter_RootMenu."-SEP6-              :"
  "
  " -----  split lines  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."&split\\ lines        :call FilterSplitLines('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."&split\\ lines   <Esc>:call FilterSplitLines('a')<CR>"
  exe "vmenu <silent> ".s:Filter_RootMenu."&split\\ lines   <Esc>:call FilterSplitLines('v')<CR>"
  "
  exe "amenu <silent>  ".s:Filter_RootMenu."-\\ split\\ s&eparator            :call FilterSplitSep()<CR>"
  exe "imenu <silent>  ".s:Filter_RootMenu."-\\ split\\ s&eparator       <Esc>:call FilterSplitSep()<CR>i"
  exe "vmenu <silent>  ".s:Filter_RootMenu."-\\ split\\ s&eparator       <Esc>:call FilterSplitSep()<CR>gv"
  "
  exe "amenu <silent>  ".s:Filter_RootMenu."-SEP7-              :"
  "
  " -----  sequence of numbers  -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."se&quence\\ of\\ numbers       :call FilterSeqOfNum()<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."se&quence\\ of\\ numbers  <Esc>:call FilterSeqOfNum()<CR>i"
  "
  exe "amenu <silent>  ".s:Filter_RootMenu."-SEP8-              :"
  "
  " -----  save buffer with time stamp  -----------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu.'save\ &buffer\ with\ time\ stamp        :call FilterSaveWithTimestamp()<CR>'
  exe "imenu <silent> ".s:Filter_RootMenu.'save\ &buffer\ with\ time\ stamp   <Esc>:call FilterSaveWithTimestamp()<CR>i'
  "
  " -----  find longest line    -------------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu."find\\ lon&gest\\ line       :call FilterLongestLine('a')<CR>"
  exe "imenu <silent> ".s:Filter_RootMenu."find\\ lon&gest\\ line  <Esc>:call FilterLongestLine('a')<CR>i"
  exe "vmenu <silent> ".s:Filter_RootMenu."find\\ lon&gest\\ line  <Esc>:call FilterLongestLine('v')<CR>"
  "
  " -----  highlight after column n    ------------------
  "
  exe "amenu <silent> ".s:Filter_RootMenu.'highlight\ after\ column\ n       :call execute( FilterHighlightColumn(), "silent!" )<CR>'
  exe "imenu <silent> ".s:Filter_RootMenu.'highlight\ after\ column\ n  <Esc>:call execute( FilterHighlightColumn(), "silent!" )<CR>i'
  "
  " -----  date and time  -------------------------------
  "
  exe "amenu  <silent>  ".s:Filter_RootMenu."-SEP9-            :"
  "
  exe "nmenu  <silent>  ".s:Filter_RootMenu.'&date          <Esc>:call FilterInsertDateAndTime("dn" )<CR>'
  exe "imenu  <silent>  ".s:Filter_RootMenu.'&date          <Esc>:call FilterInsertDateAndTime("d"  )<CR>a'
  exe "vmenu  <silent>  ".s:Filter_RootMenu.'&date         s<Esc>:call FilterInsertDateAndTime("d"  )<CR>a'
  exe "nmenu  <silent>  ".s:Filter_RootMenu.'date\ &time    <Esc>:call FilterInsertDateAndTime("dtn")<CR>'
  exe "imenu  <silent>  ".s:Filter_RootMenu.'date\ &time    <Esc>:call FilterInsertDateAndTime("dt" )<CR>a'
  exe "vmenu  <silent>  ".s:Filter_RootMenu.'date\ &time   s<Esc>:call FilterInsertDateAndTime("dt" )<CR>a'
  "
endfunction    " ----------  end of function FilterInitMenu  ----------
"
"------------------------------------------------------------------------------
"  FilterColumnateLeftRightToggle : toggle sort option : forwards <-> reverse         {{{2
"------------------------------------------------------------------------------
function! FilterColumnateLeftRightToggle()
  if s:Filter_ColumnateLeft=='yes'
    exe "aunmenu <silent>  .306 ".s:Filter_RootMenu."-\\ &left->right"
    exe "amenu   <silent>  .306 ".s:Filter_RootMenu."-\\ right->&left        :call FilterColumnateLeftRightToggle()<CR>"
    exe "imenu   <silent>  .306 ".s:Filter_RootMenu."-\\ right->&left   <Esc>:call FilterColumnateLeftRightToggle()<CR>i"
    exe "vmenu   <silent>  .306 ".s:Filter_RootMenu."-\\ right->&left   <Esc>:call FilterColumnateLeftRightToggle()<CR>gv"
    let s:Filter_ColumnateLeft='no'
  else
    exe "aunmenu <silent>  .306 ".s:Filter_RootMenu."-\\ right->&left"
    exe "amenu   <silent>  .306 ".s:Filter_RootMenu."-\\ &left->right        :call FilterColumnateLeftRightToggle()<CR>"
    exe "imenu   <silent>  .306 ".s:Filter_RootMenu."-\\ &left->right   <Esc>:call FilterColumnateLeftRightToggle()<CR>i"
    exe "vmenu   <silent>  .306 ".s:Filter_RootMenu."-\\ &left->right   <Esc>:call FilterColumnateLeftRightToggle()<CR>gv"
    let s:Filter_ColumnateLeft='yes'
  endif
endfunction    " ----------  end of function FilterColumnateLeftRightToggle  ----------
"
"------------------------------------------------------------------------------
"  FilterSortForwardsToggle : toggle sort options : forwards <-> reverse          {{{2
"------------------------------------------------------------------------------
function! FilterSortForwardsToggle()
  if s:Filter_SortForwards=='yes'
    exe "aunmenu <silent>  .105 ".s:Filter_RootMenu."-\\ &forwards->reverse"
    exe "amenu   <silent>  .105 ".s:Filter_RootMenu."-\\ reverse->&forwards        :call FilterSortForwardsToggle()<CR>"
    exe "imenu   <silent>  .105 ".s:Filter_RootMenu."-\\ reverse->&forwards   <Esc>:call FilterSortForwardsToggle()<CR>i"
    exe "vmenu   <silent>  .105 ".s:Filter_RootMenu."-\\ reverse->&forwards   <Esc>:call FilterSortForwardsToggle()<CR>gv"
    let s:Filter_SortForwards='no'
  else
    exe "aunmenu <silent>  .105 ".s:Filter_RootMenu."-\\ reverse->&forwards"
    exe "amenu   <silent>  .105 ".s:Filter_RootMenu."-\\ &forwards->reverse        :call FilterSortForwardsToggle()<CR>"
    exe "imenu   <silent>  .105 ".s:Filter_RootMenu."-\\ &forwards->reverse   <Esc>:call FilterSortForwardsToggle()<CR>i"
    exe "vmenu   <silent>  .105 ".s:Filter_RootMenu."-\\ &forwards->reverse   <Esc>:call FilterSortForwardsToggle()<CR>gv"
    let s:Filter_SortForwards='yes'
  endif
endfunction    " ----------  end of function FilterSortForwardsToggle  ----------
"
"------------------------------------------------------------------------------
"  FilterSortNumericToggle : toggle sort option : alpha <-> numeric         {{{2
"------------------------------------------------------------------------------
function! FilterSortNumericToggle()
  if s:Filter_SortAlpha=='yes'
    exe "aunmenu <silent> .110 ".s:Filter_RootMenu."-\\ &alpha->numeric"
    exe "amenu   <silent> .110 ".s:Filter_RootMenu."-\\ numeric->&alpha            :call FilterSortNumericToggle()<CR>"
    exe "imenu   <silent> .110 ".s:Filter_RootMenu."-\\ numeric->&alpha       <Esc>:call FilterSortNumericToggle()<CR>i"
    exe "vmenu   <silent> .110 ".s:Filter_RootMenu."-\\ numeric->&alpha       <Esc>:call FilterSortNumericToggle()<CR>gv"
    let s:Filter_SortAlpha='no'
  else
    exe "aunmenu <silent> .110 ".s:Filter_RootMenu."-\\ numeric->&alpha"
    exe "amenu   <silent> .110 ".s:Filter_RootMenu."-\\ &alpha->numeric            :call FilterSortNumericToggle()<CR>"
    exe "imenu   <silent> .110 ".s:Filter_RootMenu."-\\ &alpha->numeric       <Esc>:call FilterSortNumericToggle()<CR>i"
    exe "vmenu   <silent> .110 ".s:Filter_RootMenu."-\\ &alpha->numeric       <Esc>:call FilterSortNumericToggle()<CR>gv"
    let s:Filter_SortAlpha='yes'
  endif
endfunction    " ----------  end of function FilterSortNumericToggle  ----------
"
"------------------------------------------------------------------------------
"  FilterSortUniqToggle : toggle sort option : unique <-> not unique          {{{2
"------------------------------------------------------------------------------
function! FilterSortUniqToggle()
  if s:Filter_SortNotUnique=='yes'
    exe "aunmenu <silent> .115 ".s:Filter_RootMenu."-\\ not\\ unique->&unique"
    exe "amenu   <silent> .115 ".s:Filter_RootMenu."-\\ &unique->not\\ unique          :call FilterSortUniqToggle()<CR>"
    exe "imenu   <silent> .115 ".s:Filter_RootMenu."-\\ &unique->not\\ unique     <Esc>:call FilterSortUniqToggle()<CR>i"
    exe "vmenu   <silent> .115 ".s:Filter_RootMenu."-\\ &unique->not\\ unique     <Esc>:call FilterSortUniqToggle()<CR>gv"
    let s:Filter_SortNotUnique='no'
  else
    exe "aunmenu <silent> .115 ".s:Filter_RootMenu."-\\ &unique->not\\ unique"
    exe "amenu   <silent> .115 ".s:Filter_RootMenu."-\\ not\\ unique->&unique          :call FilterSortUniqToggle()<CR>"
    exe "imenu   <silent> .115 ".s:Filter_RootMenu."-\\ not\\ unique->&unique     <Esc>:call FilterSortUniqToggle()<CR>i"
    exe "vmenu   <silent> .115 ".s:Filter_RootMenu."-\\ not\\ unique->&unique     <Esc>:call FilterSortUniqToggle()<CR>gv"
    let s:Filter_SortNotUnique='yes'
  endif
endfunction    " ----------  end of function FilterSortUniqToggle  ----------
"
"------------------------------------------------------------------------------
"  FilterNumberLineZeroToggle : toggle 'number lines' option : leading zeros  <-> no leading zeros          {{{2
"------------------------------------------------------------------------------
function! FilterNumberLineZeroToggle()
  if s:Filter_NlLeadingZeros=='yes'
    exe "aunmenu<silent> .205 ".s:Filter_RootMenu."-\\ lead\\.&zeros->no\\ lead\\.zeros"
    exe "amenu  <silent> .205 ".s:Filter_RootMenu."-\\ no\\ lead\\.zeros->lead\\.&zeros        :call FilterNumberLineZeroToggle()<CR>"
    exe "imenu  <silent> .205 ".s:Filter_RootMenu."-\\ no\\ lead\\.zeros->lead\\.&zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>i"
    exe "vmenu  <silent> .205 ".s:Filter_RootMenu."-\\ no\\ lead\\.zeros->lead\\.&zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>gv"
    let s:Filter_NlLeadingZeros='no'
  else
    exe "aunmenu<silent> .205 ".s:Filter_RootMenu."-\\ no\\ lead\\.zeros->lead\\.&zeros"
    exe "amenu  <silent> .205 ".s:Filter_RootMenu."-\\ lead\\.&zeros->no\\ lead\\.zeros        :call FilterNumberLineZeroToggle()<CR>"
    exe "imenu  <silent> .205 ".s:Filter_RootMenu."-\\ lead\\.&zeros->no\\ lead\\.zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>i"
    exe "vmenu  <silent> .205 ".s:Filter_RootMenu."-\\ lead\\.&zeros->no\\ lead\\.zeros   <Esc>:call FilterNumberLineZeroToggle()<CR>gv"
    let s:Filter_NlLeadingZeros='yes'
  endif
endfunction    " ----------  end of function FilterNumberLineZeroToggle  ----------
"
"------------------------------------------------------------------------------
"  FilterNumberLineBlankLinesToggle : toggle 'number lines' option : leading zeros  <-> no leading zeros          {{{2
"------------------------------------------------------------------------------
function! FilterNumberLineBlankLinesToggle()
  if s:Filter_NlBlankLines == 'yes'
    exe "aunmenu<silent> .209 ".s:Filter_RootMenu."-\\ &blank\\ lines->no\\ blank\\ lines"
    exe "amenu  <silent> .209 ".s:Filter_RootMenu."-\\ no\\ &blank\\ lines->blank\\ lines        :call FilterNumberLineBlankLinesToggle()<CR>"
    exe "imenu  <silent> .209 ".s:Filter_RootMenu."-\\ no\\ &blank\\ lines->blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>i"
    exe "vmenu  <silent> .209 ".s:Filter_RootMenu."-\\ no\\ &blank\\ lines->blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>gv"
    let s:Filter_NlBlankLines = 'no'
  else
    exe "aunmenu<silent> .209 ".s:Filter_RootMenu."-\\ no\\ &blank\\ lines->blank\\ lines"
    exe "amenu  <silent> .209 ".s:Filter_RootMenu."-\\ &blank\\ lines->no\\ blank\\ lines        :call FilterNumberLineBlankLinesToggle()<CR>"
    exe "imenu  <silent> .209 ".s:Filter_RootMenu."-\\ &blank\\ lines->no\\ blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>i"
    exe "vmenu  <silent> .209 ".s:Filter_RootMenu."-\\ &blank\\ lines->no\\ blank\\ lines   <Esc>:call FilterNumberLineBlankLinesToggle()<CR>gv"
    let s:Filter_NlBlankLines = 'yes'
  endif
endfunction    " ----------  end of function FilterNumberLineBlankLinesToggle  ----------
"
"------------------------------------------------------------------------------
"  FilterCreateGuiMenus : create GUI menus        {{{2
"------------------------------------------------------------------------------
function! FilterCreateGuiMenus ()
  if s:Filter_MenuVisible != 1
    if s:Filter_MenuVisible != -1
      aunmenu <silent> &Tools.Load\ Textfilter
    endif
    amenu   <silent> 40.1000 &Tools.-SEP100- :
    amenu   <silent> 40.1210 &Tools.Unload\ Textfilter      :call FilterRemoveGuiMenus()<CR>
    imenu   <silent> 40.1210 &Tools.Unload\ Textfilter <C-C>:call FilterRemoveGuiMenus()<CR>
    call FilterInitMenu()
    let s:Filter_MenuVisible = 1
  endif
endfunction    " ----------  end of function FilterCreateGuiMenus  ----------

"------------------------------------------------------------------------------
"  FilterRemoveGuiMenus : remove GUI menus          {{{2
"------------------------------------------------------------------------------
function! FilterRemoveGuiMenus ()
  if s:Filter_MenuVisible == 1
    if s:Filter_RootMenu == ""
      " no submenu
    else
      exe "aunmenu <silent> ".s:Filter_RootMenu
    endif
    "
    aunmenu <silent> &Tools.Unload\ Textfilter
    amenu   <silent> 40.1000 &Tools.-SEP100- :
    amenu   <silent> 40.1210 &Tools.Load\ Textfilter      :call FilterCreateGuiMenus()<CR>
    imenu   <silent> 40.1210 &Tools.Load\ Textfilter <C-C>:call FilterCreateGuiMenus()<CR>
    "
    let s:Filter_MenuVisible = 0
  endif
endfunction    " ----------  end of function FilterRemoveGuiMenus  ----------
"
"===================================================================================
" FUNCTIONS         {{{1
"===================================================================================
"
"------------------------------------------------------------------------------
"  FilterCheckGlobal: check for global initializations          {{{2
"------------------------------------------------------------------------------
function! FilterCheckGlobal ( name )
  if exists('g:'.a:name)
    exe 'let s:'.a:name.'  = g:'.a:name
  endif
endfunction    " ----------  end of function FilterCheckGlobal  ----------
"
"------------------------------------------------------------------------------
"  FilterInput : input after a highlighted prompt         {{{2
"------------------------------------------------------------------------------
function! FilterInput ( prompt, text )
  echohl Search                                 " highlight prompt
  call  inputsave()                             " preserve typeahead
  let   retval=input( a:prompt, a:text )        " read input
  call  inputrestore()                          " restore typeahead
  echohl  None                                  " reset highlighting
  return  retval
endfunction    " ----------  end of function FilterInput  ----------
"
"------------------------------------------------------------------------------
"  FilterExpand : expand tabs to spaces         {{{2
"------------------------------------------------------------------------------
function! FilterExpand ( arg )
  let save_expandtab  = &expandtab
  exe ":set expandtab"
  "
  if a:arg=='a'
    if winheight(0) < line("$")
      let name  = FilterInput("expand whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    let cursorposition  = line(".")
    exe ":%retab"
    call  cursor( cursorposition, 1 )
  endif
  "
  if a:arg=='v'
     exe ":'<,'>retab"
  endif
  let &expandtab  = save_expandtab
endfunction    " ----------  end of function FilterExpand  ----------
"
"------------------------------------------------------------------------------
"  FilterUnexpand : unexpand : spaces to tabs         {{{2
"------------------------------------------------------------------------------
function! FilterUnexpand ( arg )
  let save_expandtab  = &expandtab
  exe ":set noexpandtab"
  "
  if a:arg=='a'
    if winheight(0) < line("$")
      let name  = FilterInput("unexpand whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    let cursorposition  = line(".")
     exe ":%retab!"
    call  cursor( cursorposition, 1 )
  endif
  "
  if a:arg=='v'
     exe ":'<,'>retab!"
  endif
  let &expandtab  = save_expandtab
endfunction    " ----------  end of function FilterUnexpand  ----------
"
"------------------------------------------------------------------------------
"  FilterSeqOfNum : sequence of numbers         {{{2
"  Return an empty string to reset to the default command
"------------------------------------------------------------------------------
function! FilterSeqOfNum ()
  "
  let prmptdef  = ' 1  1  10  "%02d"'
  if s:Filter_SeqOfNumCmd==""
    let s:Filter_SeqOfNumCmd=prmptdef                     " restore default
  endif
  let s:Filter_SeqOfNumCmd  = FilterInput( " FIRST INC LAST FORMAT : ", s:Filter_SeqOfNumCmd )
  "
  " escape some characters to avoid getting expanded by the Vim command-line processor
  "
  let cmd   =  s:Filter_SeqOfNumCmd
  "
  let param  = split( strpart( cmd, 0, match( cmd, "\"" ) ) )
  let format = strpart( cmd, match( cmd, "\"" ) )
  let format = strpart( format, 1, strlen(format)-2 ) " remove framing quotes

  if match( param[0], '\.' ) >= 0 || match( param[1], '\.' ) >= 0 || match( param[2], '\.' ) >= 0
    let ink = str2float( param[1] )
    let val = str2float( param[0] )
    let lim = str2float( param[2] )
  else
    let ink = param[1]
    let val = param[0]
    let lim = param[2]
  endif

  let pos = line('.')
  while val <= lim
    call append( pos, printf( format, val ) )
    let val += ink
    let pos += 1
  endwhile

endfunction    " ----------  end of function FilterSeqOfNum  ----------
"
"------------------------------------------------------------------------------
"  FilterNumberLines : generate numbers         {{{2
"------------------------------------------------------------------------------
function! FilterNumberLines ( arg )
  "
  let separator = '  '
  "
  if a:arg=='a'
    if winheight(0) < line("$")
      let name  = FilterInput("number whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    let pos0           = 1
    let pos1           = line("$")
    let cursorposition = line(".")
  endif
  "
  if a:arg=='v'
    let pos0           = line("'<'")
    let pos1           = line("'>'")
    let cursorposition = pos0
  endif

  let nl_width = strlen(pos1-pos0+1)
  if s:Filter_NlWidth != "0"
    let nl_width = max( [ s:Filter_NlWidth, nl_width ] )
  endif

  if s:Filter_NlLeadingZeros == 'yes'
    let format  = '%0'.nl_width.'d'
  else
    let format  = '%'.nl_width.'d'
  endif

	let number	= s:Filter_NlStart
  call  cursor( cursorposition, 1 )
  for linenumber in range( pos0, pos1 )
		let	line				= getline(linenumber)
		if line !~ '^\s*$' || s:Filter_NlBlankLines != 'no'
			let prefix      = printf( format, number )
			let line        = substitute( line, "^.*$", prefix.separator."&", ""  )
			call setline( linenumber, line )
			let	number	+= s:Filter_NlIncrement
		endif
  endfor
  call  cursor( cursorposition, 1 )

endfunction    " ----------  end of function FilterNumberLines  ----------
"
"------------------------------------------------------------------------------
"  FilterNumberLinesWidth : input number width          {{{2
"------------------------------------------------------------------------------
function! FilterNumberLinesWidth( )
  let newval = FilterInput("number width [0=auto] : ", s:Filter_NlWidth )
  if newval =~ "^\\d\\+$"
    let s:Filter_NlWidth  = newval
  endif
endfunction    " ----------  end of function FilterNumberLinesWidth  ----------
"
"------------------------------------------------------------------------------
"  FilterNumberLinesStart : input starting line number         {{{2
"------------------------------------------------------------------------------
function! FilterNumberLinesStart( )
  let newval = FilterInput("starting line number [default: 1] : ", s:Filter_NlStart )
	let	s:Filter_NlStart  = matchstr( newval, '\d\+' )
  if s:Filter_NlStart <= 0
    let s:Filter_NlStart  = 1
  endif
endfunction    " ----------  end of function FilterNumberLinesStart  ----------
"
"------------------------------------------------------------------------------
"  FilterNumberIncrement : input line number increment         {{{2
"------------------------------------------------------------------------------
function! FilterNumberIncrement( )
  let newval = FilterInput("line number increment [default: 1] : ", s:Filter_NlIncrement )
	let	s:Filter_NlIncrement  = matchstr( newval, '\d\+' )
  if s:Filter_NlIncrement <= 0
    let s:Filter_NlIncrement  = 1
  endif
endfunction    " ----------  end of function FilterNumberIncrement  ----------
"
"------------------------------------------------------------------------------
"  FilterColumnateLists : Columnate lists         {{{2
"------------------------------------------------------------------------------
function! FilterColumnateLists ( arg )
  "
  if a:arg=='a'
    if winheight(0) < line("$")
      let name = FilterInput("columnate whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    let pos0           = 1
    let pos1           = line("$")
    let cursorposition = line(".")
  endif
  "
  if a:arg=='v'
    let pos0           = line("'<")
    let pos1           = line("'>")
    let cursorposition = pos0
  endif
  "------------------------------------------------------------------------------
  " analyze
  "------------------------------------------------------------------------------
  let colmax  = []
  for linenumber in range( pos0, pos1 )
    "
    let word  = split( getline( linenumber ), s:Filter_ColumnateInputSep, 1 )
    let idx = len(colmax)
    while idx < len(word)
      let colmax  += [ 0 ]
      let idx += 1
    endwhile
    "
    let idx = 0
    for item in word
      let colmax[idx] = max( [ colmax[idx], strlen( item ) ] )
      let idx += 1
    endfor
    "
  endfor

  let emptystring = []
  let idx = 0
  for len in colmax
    let emptystring += [ ' ' ]
    while strlen( emptystring[idx] ) < colmax[idx]
      let emptystring[idx]  = emptystring[idx].emptystring[idx]
    endwhile
    let idx += 1
  endfor

  "------------------------------------------------------------------------------
  " columnate
  "------------------------------------------------------------------------------
  for linenumber in range( pos0, pos1 )
    let word    = split( getline( linenumber ), s:Filter_ColumnateInputSep, 1 )
    let newline = ''
    let idx     = 0
    for w in word
      if s:Filter_ColumnateLeft=='yes'
        let newline .= w.strpart( emptystring[idx], 0, colmax[idx]-strlen(w) ).s:Filter_ColumnateOutputSep
      else
        let newline .= strpart( emptystring[idx], 0, colmax[idx]-strlen(w) ).w.s:Filter_ColumnateOutputSep
      endif
      let idx +=1
    endfor
    let newline = substitute( newline, s:Filter_ColumnateOutputSep.'$', '', '' )
    call setline( linenumber, newline )
  endfor
  call  cursor( cursorposition, 1 )
endfunction    " ----------  end of function FilterColumnateLists  ----------
"
"------------------------------------------------------------------------------
"  FilterColumnateInputSep : read separator         {{{2
"------------------------------------------------------------------------------
function! FilterColumnateInputSep ( )
  let s:Filter_ColumnateInputSep  =
  \       FilterInput(" input separator : ", s:Filter_ColumnateInputSep )
  exe ":match Search /".s:Filter_ColumnateInputSep."/"
endfunction    " ----------  end of function FilterColumnateInputSep  ----------
"
"------------------------------------------------------------------------------
"  FilterColumnateOutputSep : read separator          {{{2
"------------------------------------------------------------------------------
function! FilterColumnateOutputSep ( )
  let s:Filter_ColumnateOutputSep  =
  \       FilterInput(" output separator : ", s:Filter_ColumnateOutputSep )
  exe ":match Search /".s:Filter_ColumnateOutputSep."/"
endfunction    " ----------  end of function FilterColumnateOutputSep  ----------
"
"------------------------------------------------------------------------------
"  FilterSort : sort lines          {{{2
"------------------------------------------------------------------------------
function! FilterSort( arg )
  "
  let sort_options=""
  if s:Filter_SortForwards=='no'
    let sort_options = "!"
  endif
  if s:Filter_SortAlpha=='no'
    let sort_options = sort_options." n"
  endif
  if s:Filter_SortNotUnique=='no'
    let sort_options = sort_options." u"
  endif
  "
  if a:arg=='a'
    if winheight(0) < line("$")
      let name  = FilterInput("sort whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    exe ":%sort".sort_options
    normal gg
  endif
  "
  if a:arg=='v'
    "
    " visual mode -- sort lines
    "
    let vmode = visualmode()
    if vmode=='v' || vmode=='V'
      exe ":'<,'>sort".sort_options
    endif
    "
    " visual mode -- sort block
    "
    if vmode== "\<c-v>"
      let pos0       = line("'<")
      let pos1       = line("'>")
      let lcol       = col("'<")
      let rcol       = col("'>")
      if lcol>rcol
        let  lcol = col("'>")
        let  rcol = col("'<")
      endif
      let blockwidth = rcol - lcol + 1
      "
      if lcol==1
        silent exe ":".pos0.",".pos1."s/\\(.\\{".blockwidth."\\}\\)\\(.*\\)/\\1 \\1\\2/"
      else
        let lcol     = lcol-1
        silent exe ":".pos0.",".pos1."s/\\(.\\{1,".lcol."\\}\\)\\(.\\{".blockwidth."\\}\\)\\(.*\\)/\\2 \\1\\2\\3/"
      endif
      "
      exe ":'<,'>sort".sort_options
      "
      let blockwidth = blockwidth + 1
      silent exe ":".pos0.",".pos1."s/^.\\{".blockwidth."\\}//"
      "
    endif
  endif
endfunction    " ----------  end of function FilterSort  ----------
"
"------------------------------------------------------------------------------
"  FilterLongestLineLength : find length of the longest line in a block         {{{2
"------------------------------------------------------------------------------
function! FilterLongestLineLength ( pos0, pos1 )
  return max( map( range( a:pos0, a:pos1), "virtcol( [v:val, '$' ] )" ) )
endfunction    " ----------  end of function FilterLongestLineLength  ----------
"
"------------------------------------------------------------------------------
"  FilterMultiLineEndFill : fill lines with spaces          {{{2
"------------------------------------------------------------------------------
function! FilterMultiLineEndFill ( arg1 )
  if a:arg1=='a'
    if winheight(0) < line("$")
      let name  = FilterInput("fill whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    let pos0=1
    let pos1=line("$")
  endif
  if a:arg1=='v'
    let  pos0      =  line("'<")
    let  pos1      =  line("'>")
  endif
  let cursorposition  = line(".")
  "
  " ----- trim whitespaces -----
  exe pos0.','.pos1.'s/\s*$//'
  " ----- find the longest line -----
  let maxlength   = FilterLongestLineLength(pos0,pos1)
  " ----- fill lines with spaces -----
  call  cursor( pos0, 1 )
  for linenumber in range( pos0, pos1 )
    let diff = maxlength-virtcol("$")
    if diff > 0
      exe "normal ".diff."A "
    endif
    normal j
  endfor
  " ----- back to the beginning of the marked block -----
  call  cursor( cursorposition, 1 )
endfunction    " ----------  end of function FilterMultiLineEndFill  ----------
"
"------------------------------------------------------------------------------
"  FilterCompressEmptyLines : compress empty lines          {{{2
"------------------------------------------------------------------------------
function! FilterCompressEmptyLines ( arg1 )
  if a:arg1=='a'
    if winheight(0) < line("$")
      let name  = FilterInput("whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    let pos0=1
    let pos1=line("$")
  endif
  if a:arg1=='v'
    let  pos0      =  line("'<")
    let  pos1      =  line("'>")
  endif
  let cursorposition  = line(".")
  "
  " one line infile
  if pos1-pos0 == 0
    return
  endif
  "
  " two lines infile
  if pos1-pos0 == 1
    silent! exe pos0.','.pos1.'s/^\s*\n\s*$//'
    return
  endif
  silent! exe pos0.','.pos1.'s/^\(\s*\n\)\{2,\}/\r/'
  " special treatment for the last 2 lines:
  " the last line does not terminate with a \n
  if getline(line("$")-1)=="" && getline("$")!~'\S\+'
    exe ":$,$d"
  endif
  " ----- back to the beginning of the marked block -----
  call  cursor( cursorposition, 1 )
endfunction    " ----------  end of function FilterCompressEmptyLines  ----------
"
"------------------------------------------------------------------------------
"  FilterMultiLineTrim : trim lines         {{{2
"------------------------------------------------------------------------------
function! FilterMultiLineTrim ( arg1 )
  "
  if a:arg1=='a'
    let pos0 = 1
    let pos1 = line("$")
    if winheight(0) < line("$")
      let name  = FilterInput("trim whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
  endif
  "
  if a:arg1=='v'
    let  pos0 = line("'<")
    let  pos1 = line("'>")
  endif
  "
  let cursorposition  = line(".")
  " ----- trim whitespaces -----
  try
    exe pos0.','.pos1.'s/\s\+$//'
  catch /.*/
    echohl Search | echomsg "No trailing whitespaces found" | echohl None
  endtry
  " ----- back to the beginning of the marked block -----
  call  cursor( cursorposition, 1 )
endfunction    " ----------  end of function FilterMultiLineTrim  ----------

"------------------------------------------------------------------------------
"  FilterSplitLines : split lines         {{{2
"------------------------------------------------------------------------------
function! FilterSplitLines ( arg1 )
  "
  if a:arg1=='a'
    let pos0 = line(".")
    let pos1 = pos0
  endif
  "
  if a:arg1=='v'
    let  pos0 = line("'<")
    let  pos1 = line("'>")
  endif
  "
  let cursorposition  = line(".")
  " ----- split lines -----
  while s:Filter_SplitSeparator==''
    call FilterSplitSep()
  endwhile
  silent! exe pos0.','.pos1.'s/'.s:Filter_SplitSeparator.'/\r/g'
  " ----- back to the beginning of the marked block -----
  call  cursor( cursorposition, 1 )
endfunction    " ----------  end of function FilterSplitLines  ----------
"
"------------------------------------------------------------------------------
"  FilterSplitSep : read separator          {{{2
"------------------------------------------------------------------------------
function! FilterSplitSep ( )
  let s:Filter_SplitSeparator  =
  \       FilterInput(" split separator : ", s:Filter_SplitSeparator )
  exe ":match Search /".s:Filter_SplitSeparator."/"
endfunction    " ----------  end of function FilterSplitSep  ----------
"
"------------------------------------------------------------------------------
"  FilterLeftJustify : left justify         {{{2
"------------------------------------------------------------------------------
function! FilterLeftJustify ( arg1 )
  let cursorposition  = line(".")
  if a:arg1=='a'
    let pos0 = 1
    let pos1 = line("$")
    if winheight(0) < line("$")
      let name  = FilterInput("left justify whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
  endif
  if a:arg1=='v'
    let  pos0 = line("'<")
    let  pos1 = line("'>")
    let  lcol = virtcol("'<")
    let  rcol = virtcol("'>")
    if lcol>rcol
      let  lcol = virtcol("'>")
      let  rcol = virtcol("'<")
    endif
  endif
  "
  let save_expandtab  = &expandtab
  exe ":set expandtab"
  exe ":".pos0.",".pos1."retab"
  let &expandtab  = save_expandtab
  "
  " ----- left justify block ------------------
  "
  if a:arg1=='v' && visualmode()== "\<c-v>" && lcol<rcol
    exe pos0
    for linenumber in range( pos0, pos1 )
      let curline = getline(linenumber)
      if curline =~ '\S\+'           " do not process empty lines
        let head  = strpart( curline, 0, lcol-1 )
        let block = strpart( curline, lcol-1, rcol-lcol+1 )
        let tail  = strpart( curline, rcol )
        let block = substitute( block, '^\(\s*\)\(.*\)', '\2\1' , "" )
        call setline( linenumber, head.block.tail )
      endif
    endfor
  "
  " ----- left justify lines ------------------
  "
  else
    exe pos0.','.pos1.'s/^\s*//'
  endif
  " ----- back to the beginning of the marked block -----
  call  cursor( cursorposition, 1 )
endfunction    " ----------  end of function FilterLeftJustify  ----------
"
"------------------------------------------------------------------------------
"  FilterRightJustify : right justify         {{{2
"------------------------------------------------------------------------------
function! FilterRightJustify ( arg1 )
  if a:arg1=='a'
    if winheight(0) < line("$")
      let name  = FilterInput("right justify whole file [y/n/Esc] : ", "y" )
      if name != "y"
        return
      endif
    endif
    let pos0=1
    let pos1=line("$")
  endif
  if a:arg1=='v'
    let  pos0 = line("'<")
    let  pos1 = line("'>")
    let  lcol = virtcol("'<")
    let  rcol = virtcol("'>")
    if lcol>rcol
      let  lcol = virtcol("'>")
      let  rcol = virtcol("'<")
    endif
  endif
  let cursorposition  = line(".")
  "
  let save_expandtab  = &expandtab
  exe ":set expandtab"
  exe ":".pos0.",".pos1."retab"
  let &expandtab  = save_expandtab
  "
  " ----- right justify block ------------------
  "
  if a:arg1=='v' && visualmode()== "\<c-v>" && lcol<rcol
    exe pos0
    let blockwidth  = rcol-lcol+1
    for linenumber in range( pos0, pos1 )
      let curline = getline( linenumber )
      if curline =~ '\S\+'           " do not process empty lines
        let head    = strpart( curline, 0, lcol-1 )
        let block   = strpart( curline, lcol-1, blockwidth )
        let tail    = strpart( curline, rcol )
        " fill blocks which are to short
        while strlen( block ) < blockwidth
          let block = block.' '
        endwhile
        let block =substitute( block, '\(.\{-}\)\(\s*\)$', '\2\1' , "" )
        call setline( linenumber, head.block.tail )
      endif
    endfor
  "
  " ----- right justify lines ------------------
  "
  else
    " ----- trim whitespaces -----
    exe pos0.','.pos1.'s/\s*$//'
    " ----- find the longest line -----
    let maxlength  = FilterLongestLineLength(pos0,pos1)
    exe pos0
    for linenumber in range( pos0, pos1 )
      let diff = maxlength-virtcol("$")
      " ----- right justify : prepend spaces -----
      if diff>0
        exe "normal ".diff."I "
      endif
      normal j
    endfor
  endif
  " ----- back to the beginning of the marked block -----
  call  cursor( cursorposition, 1 )
endfunction    " ----------  end of function FilterRightJustify  ----------
"
"------------------------------------------------------------------------------
"  FilterLongestLine : find longest line          {{{2
"------------------------------------------------------------------------------
function! FilterLongestLine( mode )
  if a:mode=="a"
    let firstline   = line(".")
    let lastline    = line("$")
  endif
  if a:mode=="v"
    let firstline   = line("'<")
    let lastline    = line("'>")
  endif
  let longline    = firstline
  let maxlength   = 0
  for linenumber in range( firstline, lastline )
    call  cursor( linenumber, 1 )
    let vc  = virtcol("$")
    if maxlength == vc
      let equline   = equline+1
    endif
    if maxlength < vc
      let maxlength = vc
      let longline  = linenumber
      let equline   = 0
    endif
  endfor
  call  cursor( longline, 1 )
  let maxlength = maxlength-1
  redraw
  echohl Search                        " highlight prompt
  echo "range ".firstline."-".lastline." : longest line has number ".longline.", length ".maxlength
        \ ."  (".equline." line(s) of equal length below)"
  echohl None                          " reset highlighting
endfunction    " ----------  end of function FilterLongestLine  ----------
"
"------------------------------------------------------------------------------
"  FilterHighlightColumn : highlight after column n         {{{2
"------------------------------------------------------------------------------
function! FilterHighlightColumn ()
  let prompt  = "highlight after column : "
  if exists("b:FilterHighlightCol")
    let b:FilterHighlightCol= FilterInput( prompt, b:FilterHighlightCol )
  else
    let b:FilterHighlightCol= FilterInput( prompt , s:Filter_HighlightAfterCol )
  endif
  if b:FilterHighlightCol =~ '^\s*\d\+\s*$'
    let b:FilterHighlightCol = substitute( b:FilterHighlightCol, '^\s*', '' , "" )
    let b:FilterHighlightCol = substitute( b:FilterHighlightCol, '\s*$', '' , "" )
    return  '/\%>'.b:FilterHighlightCol.'v.\+$'
    return
  else
    let b:FilterHighlightCol= ""
    return ""
  endif
endfunction   " ---------- end of function  FilterHighlightColumn  ----------
"
"------------------------------------------------------------------------------
"  FilterSaveWithTimestamp : save buffer with time stamp / preserve file extension          {{{2
"------------------------------------------------------------------------------
function! FilterSaveWithTimestamp ()
  if expand("%") == ""
    redraw
    echohl WarningMsg | echo " no file name " | echohl None
    return
  endif
  let newname  = escape( expand("%:p:r"), s:Filter_EscFilename ).".".strftime(s:Filter_TimestampFormat)
  let ext  = expand("%:e")
  if ext != ''
    let newname   = newname.'.'.ext
  endif
  exe ":write ".newname
  "
  redraw
  echohl Search | echomsg "file  '".newname."'  written " | echohl None
endfunction   " ---------- end of function  FilterSaveWithTimestamp  ----------
"
"------------------------------------------------------------------------------
"  FilterInsertDateAndTime : insert date and time         {{{2
"------------------------------------------------------------------------------
function! FilterInsertDateAndTime ( format )
  if a:format == 'dn' || a:format == 'dtn'      " normal mode
    call FilterDateAndTime(a:format)
  else                                          " case 'd' or 'dt'
    if col(".") > 1
      exe 'normal a'.FilterDateAndTime(a:format)
    else
      exe 'normal i'.FilterDateAndTime(a:format)
    endif
  endif
endfunction    " ----------  end of function FilterInsertDateAndTime  ----------

"------------------------------------------------------------------------------
"  FilterDateAndTime : generate date and time         {{{2
"------------------------------------------------------------------------------
function! FilterDateAndTime ( format )
  "
  if a:format == 'd'
    return strftime( s:Filter_FormatDate )
  endif
    "
  if a:format == 't'
    return strftime( s:Filter_FormatTime )
  endif
    "
  if a:format == 'dt'
    return strftime( s:Filter_FormatDate ).' '.strftime( s:Filter_FormatTime )
  endif
  "
  " ---------- 'dn' and 'dtn' ----------
  "
  let date = strftime( s:Filter_FormatDate )
  if a:format == 'dtn'
    let date .= ' '.strftime( s:Filter_FormatTime )
  endif
  " ---------- build regex, e.g. '\d\d\.\d\d\.\d\d \d\d\:\d\d\:\d\d' ----------
  let rgx   = substitute( date, '\a', 'a', 'g' )     " replace alphabetic characters
  let rgx   = substitute( rgx,  '\d', 'd', 'g' )     " and replace digits
  let rgx   = escape( rgx, 'ad.' )                   " by \a and \d
  call setline( line('.'), substitute( getline('.'), rgx, date, '' ) )
  "
endfunction    " ----------  end of function FilterDateAndTime  ----------
"
"===================================================================================
" INITIALIZATION          {{{1
"===================================================================================
"
"------------------------------------------------------------------------------
"  Look for global variables (if any), to override some defaults.
"------------------------------------------------------------------------------
call FilterCheckGlobal("Filter_FormatDate       ")
call FilterCheckGlobal("Filter_FormatTime       ")
call FilterCheckGlobal("Filter_HighlightAfterCol")
call FilterCheckGlobal("Filter_LoadMenus        ")
call FilterCheckGlobal("Filter_RootMenu         ")
call FilterCheckGlobal("Filter_TimestampFormat  ")
"
if s:Filter_RootMenu == ""
  let s:Filter_RootMenu = '&Textfilter.'       " use the default
endif
"
"------------------------------------------------------------------------------
"  show / hide the menus
"  define key mappings (gVim only)
"------------------------------------------------------------------------------
"
if has("gui_running")
  "
  if s:Filter_LoadMenus == 'yes'
    call FilterCreateGuiMenus()
  endif
  "
endif
"
" vim: foldmethod=marker foldlevel=1
